==> ms-users-1.4.0.tgz: package/lib/utils/send-email.js <==
'use strict';

const Errors = require('common-errors');
const Promise = require('bluebird'); // jshint ignore: line
const uuid = require('node-uuid');
const redisKey = require('../utils/key.js');
const crypto = require('crypto');
const URLSafeBase64 = require('urlsafe-base64');
const render = require('ms-mailer-templates');

var _require = require('../actions/updatePassword.js');

const updatePassword = _require.updatePassword;

const generatePassword = require('password-generator');

/**
 * Throttled error
 * @param  {Mixed}  reply
 */
function isThrottled(compare) {
  return function comparator(reply) {
    if (!!reply === compare) {
      throw new Errors.HttpStatusError(429, 'We\'ve already sent you an email, if it doesn\'t come - please try again in a little while or send us an email'); // eslint-disable-line
    }
  };
}

/**
 * Encrypts buffer using alg and secret
 * @param  {String} algorithm
 * @param  {String} secret
 * @param  {Buffer} buffer
 * @return {Buffer}
 */
exports.encrypt = function encrypt(algorithm, secret, buffer) {
  const cipher = crypto.createCipher(algorithm, secret);
  const crypted = Buffer.concat([cipher.update(buffer), cipher.final()]);
  return crypted;
};

/**
 * Decrypts buffer using algoruthm and secret
 * @param  {String} algorithm
 * @param  {String} secret
 * @param  {Buffer} buffer
 * @return {Buffer}
 */
exports.decrypt = function decrypt(algorithm, secret, buffer) {
  const decipher = crypto.createDecipher(algorithm, secret);
  const dec = Buffer.concat([decipher.update(buffer), decipher.final()]);
  return dec;
};

/**
 * Generates complete link
 * @param  {Object} server
 * @param  {String} path
 * @return {String}
 */
exports.generateLink = function generateLink(server, path) {
  const proto = server.proto;
  let port = server.port;

  if (proto === 'http' && +port === 80 || proto === 'https' && +port === 443) {
    port = '';
  } else {
    port = `:${ port }`;
  }

  return `${ proto }://${ server.host + port + path }`;
};

/**
 * Safely decodes
 * @param  {String} algorithm
 * @param  {String} secret
 * @param  {String} string
 * @return {Promise}
 */
exports.safeDecode = function safeDecode(algorithm, secret, string) {
  return Promise.try(function decode() {
    return JSON.parse(exports.decrypt(algorithm, secret, URLSafeBase64.decode(string)));
  }).bind(this).catch(function remapError(err) {
    this.log.warn('cant decode token', err);
    throw new Errors.HttpStatusError(403, 'could not decode token');
  });
};

/**
 * Send an email with appropriate content
 *
 * @param  {String} email
 * @param  {String} type
 * @return {Promise}
 */
exports.send = function sendEmail(email) {
  let type = arguments.length <= 1 || arguments[1] === undefined ? 'activate' : arguments[1];
  let wait = arguments.length <= 2 || arguments[2] === undefined ? false : arguments[2];
  const redis = this.redis;
  const config = this.config;
  const mailer = this.mailer;
  const validation = config.validation;
  const server = config.server;
  const ttl = validation.ttl;
  const throttle = validation.throttle;
  const subjects = validation.subjects;
  const senders = validation.senders;
  const paths = validation.paths;
  const secret = validation.secret;
  const algorithm = validation.algorithm;
  const mailingAccount = validation.email; // eslint-disable-line

  // method specific stuff

  const throttleEmailsKey = redisKey('vthrottle-' + type, email);
  const activationSecret = uuid.v4();
  const logger = this.log.child({ action: 'sendEmail', email: email });

  return redis.get(throttleEmailsKey).then(isThrottled(true)).then(function generateContent() {
    // generate context
    const context = {};
    const templateName = validation.templates[type] || type;

    switch (type) {
      case 'activate':
      case 'reset':
        // generate secret
        const str = new Buffer(JSON.stringify({ email: email, token: activationSecret }));
        const enc = exports.encrypt(algorithm, secret, str);
        context.qs = '?q=' + URLSafeBase64.encode(enc);
        context.link = exports.generateLink(server, paths[type]);
        break;
      case 'password':
        context.password = generatePassword(config.pwdReset.length, config.pwdReset.memorable);
        break;
      default:
        throw new Errors.InvalidOperationError(`${ type } action is not supported`);
    }

    return Promise.props({
      context: context,
      emailTemplate: render(templateName, context)
    });
  }).tap(data => {
    const context = data.context;

    // in case we need to setup a new password

    if (type === 'password') {
      return updatePassword.call(this, email, context.password);
    }

    const throttleArgs = [throttleEmailsKey, 1, 'NX'];
    if (throttle > 0) {
      throttleArgs.splice(2, 0, 'EX', throttle);
    }
    return redis.set(throttleArgs).then(isThrottled(false)).then(function updateSecret() {
      const secretKey = redisKey('vsecret-' + type, activationSecret);
      const args = [secretKey, email];
      if (ttl > 0) {
        args.push('EX', ttl);
      }
      return redis.set(args);
    });
  }).then(function definedSubjectAndSend(_ref) {
    let context = _ref.context;
    let emailTemplate = _ref.emailTemplate;

    const mail = {
      subject: subjects[type] || '',
      from: senders[type] || 'noreply <support@example.com>',
      to: email,
      html: emailTemplate
    };

    const mailSent = mailer.send(mailingAccount, mail).return({ sent: true, context: context }).catch(function mailingFailed(err) {
      logger.warn('couldn\'t send email', err);
      return { sent: false, context: context, err: err };
    });

    if (wait) {
      return mailSent;
    }

    return {
      queued: true,
      context: context
    };
  });
};

/**
 * Verifies token
 * @param  {String}  string
 * @param  {String}  namespace
 * @param  {Boolean} expire
 * @return {Promise}
 */
exports.verify = function verifyToken(string) {
  let namespace = arguments.length <= 1 || arguments[1] === undefined ? 'activate' : arguments[1];
  let expires = arguments[2];
  const redis = this.redis;
  const config = this.config;
  const validation = config.validation;
  const validationSecret = validation.secret;
  const algorithm = validation.algorithm;

  return exports.safeDecode.call(this, algorithm, validationSecret, string).then(function inspectResult(message) {
    const email = message.email;
    const token = message.token;

    if (!email || !token) {
      const msg = 'Decoded token misses references to email and/or secret';
      throw new Errors.HttpStatusError(403, msg);
    }

    const secretKey = redisKey('vsecret-' + namespace, token);
    return redis.get(secretKey).then(function inspectAssociatedData(associatedEmail) {
      if (!associatedEmail) {
        throw new Errors.HttpStatusError(404, 'token expired or is invalid');
      }

      if (associatedEmail !== email) {
        throw new Errors.HttpStatusError(412, 'associated email doesn\'t match token');
      }

      if (expires) {
        return redis.del(secretKey);
      }
    }).return(email);
  });
};

==> ms-users-1.10.1.tgz: package/lib/utils/send-email.js <==
'use strict';

const Errors = require('common-errors');
const Promise = require('bluebird'); // jshint ignore: line
const uuid = require('node-uuid');
const redisKey = require('../utils/key.js');
const crypto = require('crypto');
const URLSafeBase64 = require('urlsafe-base64');
const render = require('ms-mailer-templates');

var _require = require('../actions/updatePassword.js');

const updatePassword = _require.updatePassword;

const generatePassword = require('password-generator');

var _require2 = require('../constants.js');

const MAIL_ACTIVATE = _require2.MAIL_ACTIVATE;
const MAIL_RESET = _require2.MAIL_RESET;
const MAIL_PASSWORD = _require2.MAIL_PASSWORD;

/**
 * Throttled error
 * @param  {Mixed}  reply
 */

function isThrottled(compare) {
  return function comparator(reply) {
    if (!!reply === compare) {
      throw new Errors.HttpStatusError(429, 'We\'ve already sent you an email, if it doesn\'t come - please try again in a little while or send us an email'); // eslint-disable-line
    }
  };
}

/**
 * Encrypts buffer using alg and secret
 * @param  {String} algorithm
 * @param  {String} secret
 * @param  {Buffer} buffer
 * @return {Buffer}
 */
exports.encrypt = function encrypt(algorithm, secret, buffer) {
  const cipher = crypto.createCipher(algorithm, secret);
  const crypted = Buffer.concat([cipher.update(buffer), cipher.final()]);
  return crypted;
};

/**
 * Decrypts buffer using algoruthm and secret
 * @param  {String} algorithm
 * @param  {String} secret
 * @param  {Buffer} buffer
 * @return {Buffer}
 */
exports.decrypt = function decrypt(algorithm, secret, buffer) {
  const decipher = crypto.createDecipher(algorithm, secret);
  const dec = Buffer.concat([decipher.update(buffer), decipher.final()]);
  return dec;
};

/**
 * Generates complete link
 * @param  {Object} server
 * @param  {String} path
 * @return {String}
 */
exports.generateLink = function generateLink(server, path) {
  const proto = server.proto;
  let port = server.port;


  if (proto === 'http' && +port === 80 || proto === 'https' && +port === 443) {
    port = '';
  } else {
    port = `:${ port }`;
  }

  return `${ proto }://${ server.host + port + path }`;
};

/**
 * Safely decodes
 * @param  {String} algorithm
 * @param  {String} secret
 * @param  {String} string
 * @return {Promise}
 */
exports.safeDecode = function safeDecode(algorithm, secret, string) {
  return Promise.try(function decode() {
    return JSON.parse(exports.decrypt(algorithm, secret, URLSafeBase64.decode(string)));
  }).bind(this).catch(function remapError(err) {
    this.log.warn('cant decode token', err);
    throw new Errors.HttpStatusError(403, 'could not decode token');
  });
};

/**
 * Send an email with appropriate content
 *
 * @param  {String} email
 * @param  {String} type
 * @return {Promise}
 */
exports.send = function sendEmail(email) {
  let type = arguments.length <= 1 || arguments[1] === undefined ? MAIL_ACTIVATE : arguments[1];
  let wait = arguments.length <= 2 || arguments[2] === undefined ? false : arguments[2];
  const redis = this.redis;
  const config = this.config;
  const mailer = this.mailer;
  const validation = config.validation;
  const server = config.server;
  const ttl = validation.ttl;
  const throttle = validation.throttle;
  const subjects = validation.subjects;
  const senders = validation.senders;
  const paths = validation.paths;
  const secret = validation.secret;
  const algorithm = validation.algorithm;
  const mailingAccount = validation.email; // eslint-disable-line

  // method specific stuff

  const throttleEmailsKey = redisKey(`vthrottle-${ type }`, email);
  const activationSecret = uuid.v4();
  const logger = this.log.child({ action: 'sendEmail', email: email });

  return redis.get(throttleEmailsKey).then(isThrottled(true)).then(function generateContent() {
    // generate context
    const context = {};
    const templateName = validation.templates[type] || type;

    switch (type) {
      case MAIL_ACTIVATE:
      case MAIL_RESET:
        {
          // generate secret
          const str = new Buffer(JSON.stringify({ email: email, token: activationSecret }));
          const enc = exports.encrypt(algorithm, secret, str);
          context.qs = `?q=${ URLSafeBase64.encode(enc) }`;
          context.link = exports.generateLink(server, paths[type]);
          break;
        }
      case MAIL_PASSWORD:
        context.password = generatePassword(config.pwdReset.length, config.pwdReset.memorable);
        break;
      default:
        throw new Errors.InvalidOperationError(`${ type } action is not supported`);
    }

    return Promise.props({
      context: context,
      emailTemplate: render(templateName, context)
    });
  }).tap(data => {
    const context = data.context;

    // in case we need to setup a new password

    if (type === 'password') {
      return updatePassword.call(this, email, context.password);
    }

    const throttleArgs = [throttleEmailsKey, 1, 'NX'];
    if (throttle > 0) {
      throttleArgs.splice(2, 0, 'EX', throttle);
    }
    return redis.set(throttleArgs).then(isThrottled(false)).then(function updateSecret() {
      const secretKey = redisKey(`vsecret-${ type }`, activationSecret);
      const args = [secretKey, email];
      if (ttl > 0) {
        args.push('EX', ttl);
      }
      return redis.set(args);
    });
  }).then(function definedSubjectAndSend(_ref) {
    let context = _ref.context;
    let emailTemplate = _ref.emailTemplate;

    const mail = {
      subject: subjects[type] || '',
      from: senders[type] || 'noreply <support@example.com>',
      to: email,
      html: emailTemplate
    };

    const mailSent = mailer.send(mailingAccount, mail).return({ sent: true, context: context }).catch(function mailingFailed(err) {
      logger.warn('couldn\'t send email', err);
      return { sent: false, context: context, err: err };
    });

    if (wait) {
      return mailSent;
    }

    return {
      queued: true,
      context: context
    };
  });
};

/**
 * Verifies token
 * @param  {String}  string
 * @param  {String}  namespace
 * @param  {Boolean} expire
 * @return {Promise}
 */
exports.verify = function verifyToken(string) {
  let namespace = arguments.length <= 1 || arguments[1] === undefined ? MAIL_ACTIVATE : arguments[1];
  let expires = arguments[2];
  const redis = this.redis;
  const config = this.config;
  const validation = config.validation;
  const validationSecret = validation.secret;
  const algorithm = validation.algorithm;


  return exports.safeDecode.call(this, algorithm, validationSecret, string).then(function inspectResult(message) {
    const email = message.email;
    const token = message.token;


    if (!email || !token) {
      const msg = 'Decoded token misses references to email and/or secret';
      throw new Errors.HttpStatusError(403, msg);
    }

    const secretKey = redisKey(`vsecret-${ namespace }`, token);
    return redis.get(secretKey).then(function inspectAssociatedData(associatedEmail) {
      if (!associatedEmail) {
        throw new Errors.HttpStatusError(404, 'token expired or is invalid');
      }

      if (associatedEmail !== email) {
        throw new Errors.HttpStatusError(412, 'associated email doesn\'t match token');
      }

      if (expires) {
        return redis.del(secretKey);
      }

      return null;
    }).return(email);
  });
};
